@@ -71,7 +71,7 @@ enum cmd_ops;
 int nft_cache_evaluate(struct nft_ctx *nft, struct list_head *cmds,
 		       struct list_head *msgs, struct nft_cache_filter *filter,
 		       unsigned int *flags);
-int nft_cache_update(struct nft_ctx *ctx, enum cmd_ops cmd,
+int nft_cache_update(struct nft_ctx *ctx, unsigned int flags,
 		     struct list_head *msgs,
 		     const struct nft_cache_filter *filter);
 bool nft_cache_needs_update(struct nft_cache *cache);
