@@ -11,7 +11,6 @@ struct nlmsghdr;
 struct rule;
 struct set;
 struct obj;
-struct flowtable;
 struct stmt;
 struct symbol_table;
 struct table;
@@ -75,6 +74,7 @@ json_t *payload_stmt_json(const struct stmt *stmt, struct output_ctx *octx);
 json_t *exthdr_stmt_json(const struct stmt *stmt, struct output_ctx *octx);
 json_t *quota_stmt_json(const struct stmt *stmt, struct output_ctx *octx);
 json_t *ct_stmt_json(const struct stmt *stmt, struct output_ctx *octx);
+json_t *last_stmt_json(const struct stmt *stmt, struct output_ctx *octx);
 json_t *limit_stmt_json(const struct stmt *stmt, struct output_ctx *octx);
 json_t *fwd_stmt_json(const struct stmt *stmt, struct output_ctx *octx);
 json_t *notrack_stmt_json(const struct stmt *stmt, struct output_ctx *octx);
@@ -84,7 +84,6 @@ json_t *nat_stmt_json(const struct stmt *stmt, struct output_ctx *octx);
 json_t *reject_stmt_json(const struct stmt *stmt, struct output_ctx *octx);
 json_t *counter_stmt_json(const struct stmt *stmt, struct output_ctx *octx);
 json_t *set_stmt_json(const struct stmt *stmt, struct output_ctx *octx);
-json_t *map_stmt_json(const struct stmt *stmt, struct output_ctx *octx);
 json_t *log_stmt_json(const struct stmt *stmt, struct output_ctx *octx);
 json_t *objref_stmt_json(const struct stmt *stmt, struct output_ctx *octx);
 json_t *meter_stmt_json(const struct stmt *stmt, struct output_ctx *octx);
@@ -113,8 +112,6 @@ void monitor_print_element_json(struct netlink_mon_handler *monh,
 				const char *cmd, struct set *s);
 void monitor_print_obj_json(struct netlink_mon_handler *monh,
 			    const char *cmd, struct obj *o);
-void monitor_print_flowtable_json(struct netlink_mon_handler *monh,
-				  const char *cmd, struct flowtable *ft);
 void monitor_print_rule_json(struct netlink_mon_handler *monh,
 			     const char *cmd, struct rule *r);
 
@@ -181,6 +178,7 @@ STMT_PRINT_STUB(payload)
 STMT_PRINT_STUB(exthdr)
 STMT_PRINT_STUB(quota)
 STMT_PRINT_STUB(ct)
+STMT_PRINT_STUB(last)
 STMT_PRINT_STUB(limit)
 STMT_PRINT_STUB(fwd)
 STMT_PRINT_STUB(notrack)
@@ -190,7 +188,6 @@ STMT_PRINT_STUB(nat)
 STMT_PRINT_STUB(reject)
 STMT_PRINT_STUB(counter)
 STMT_PRINT_STUB(set)
-STMT_PRINT_STUB(map)
 STMT_PRINT_STUB(log)
 STMT_PRINT_STUB(objref)
 STMT_PRINT_STUB(meter)
@@ -255,13 +252,6 @@ static inline void monitor_print_obj_json(struct netlink_mon_handler *monh,
 	/* empty */
 }
 
-static inline void
-monitor_print_flowtable_json(struct netlink_mon_handler *monh,
-			     const char *cmd, struct flowtable *ft)
-{
-	/* empty */
-}
-
 static inline void monitor_print_rule_json(struct netlink_mon_handler *monh,
 					   const char *cmd, struct rule *r)
 {
