@@ -110,6 +110,47 @@ struct expr *get_set_intervals(const struct set *set, const struct expr *init)
 	return new_init;
 }
 
+static struct expr *get_set_interval_find(const struct set *cache_set,
+					  struct expr *left,
+					  struct expr *right)
+{
+	const struct set *set = cache_set;
+	struct expr *range = NULL;
+	struct expr *i;
+	mpz_t val;
+
+	mpz_init2(val, set->key->len);
+
+	list_for_each_entry(i, &set->init->expressions, list) {
+		switch (i->key->etype) {
+		case EXPR_VALUE:
+			if (expr_basetype(i->key)->type != TYPE_STRING)
+				break;
+			/* string type, check if its a range (wildcard), so
+			 * fall through.
+			 */
+		case EXPR_PREFIX:
+		case EXPR_RANGE:
+			range_expr_value_low(val, i);
+			if (left && mpz_cmp(left->key->value, val))
+				break;
+
+			range_expr_value_high(val, i);
+			if (right && mpz_cmp(right->key->value, val))
+				break;
+
+			range = expr_clone(i->key);
+			goto out;
+		default:
+			break;
+		}
+	}
+out:
+	mpz_clear(val);
+
+	return range;
+}
+
 static struct expr *expr_value(struct expr *expr)
 {
 	switch (expr->etype) {
@@ -124,48 +165,6 @@ static struct expr *expr_value(struct expr *expr)
 	}
 }
 
-static struct expr *get_set_interval_find(const struct set *cache_set,
-					  struct expr *left,
-					  struct expr *right)
-{
-	const struct set *set = cache_set;
-	struct expr *range = NULL;
-	struct expr *i, *key;
-	mpz_t val;
-
-	mpz_init2(val, set->key->len);
-
-	list_for_each_entry(i, &set->init->expressions, list) {
-		key = expr_value(i);
-		switch (key->etype) {
-		case EXPR_VALUE:
-			if (expr_basetype(i->key)->type != TYPE_STRING)
-				break;
-			/* string type, check if its a range (wildcard), so
-			 * fall through.
-			 */
-		case EXPR_PREFIX:
-		case EXPR_RANGE:
-			range_expr_value_low(val, i);
-			if (left && mpz_cmp(expr_value(left)->value, val))
-				break;
-
-			range_expr_value_high(val, i);
-			if (right && mpz_cmp(expr_value(right)->value, val))
-				break;
-
-			range = expr_clone(i);
-			goto out;
-		default:
-			break;
-		}
-	}
-out:
-	mpz_clear(val);
-
-	return range;
-}
-
 static struct expr *__expr_to_set_elem(struct expr *low, struct expr *expr)
 {
 	struct expr *elem = set_elem_expr_alloc(&low->location, expr);
@@ -425,7 +424,7 @@ next:
 			mpz_clear(range);
 
 			r2 = list_entry(r2_next, typeof(*r2), list);
-			compound_expr_remove(expr_value(start), r1);
+			compound_expr_remove(start, r1);
 
 			if (free_r1)
 				expr_free(r1);
@@ -473,7 +472,7 @@ static struct expr *interval_to_string(struct expr *low, struct expr *i, const m
 
 	expr = constant_expr_alloc(&low->location, low->dtype,
 				   BYTEORDER_HOST_ENDIAN,
-				   len * BITS_PER_BYTE, data);
+				   (str_len + 1) * BITS_PER_BYTE, data);
 
 	return __expr_to_set_elem(low, expr);
 }
@@ -631,10 +630,8 @@ void interval_map_decompose(struct expr *set)
 	expr_free(i);
 
 out:
-	if (catchall) {
-		catchall->flags |= EXPR_F_KERNEL;
+	if (catchall)
 		compound_expr_add(set, catchall);
-	}
 
 	xfree(ranges);
 	xfree(elements);
